@@ -85,7 +85,7 @@ AsnOid *get_esi(int fd)
   return name;
 }
 
-void delete_nsap(void)
+void delete_nsap(int itf)
 {
   struct atmif_sioc req;
   int fd;
@@ -93,16 +93,16 @@ void delete_nsap(void)
   if ((fd = socket(AF_ATMSVC, SOCK_DGRAM, 0)) < 0)
     diag(COMPONENT, DIAG_FATAL, "socket: %s", strerror(errno));
 
-  req.number = atm_itf;
+  req.number = itf;
   req.arg = NULL;
 
-  if(ioctl(fd, SIOCSIFATMADDR, &req) < 0)
-    diag(COMPONENT, DIAG_FATAL, "ioctl SIOCSIFATMADDR: %s", strerror(errno));
+  if(ioctl(fd, ATM_RSTADDR, &req) < 0)
+    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_RSTADDR: %s", strerror(errno));
 
   close(fd);
 }
 
-void add_nsap(AsnOid *netprefix, AsnOid *esi)
+void add_nsap(int itf, AsnOid *netprefix, AsnOid *esi)
 {
   struct atmif_sioc req;
   struct sockaddr_atmsvc addr;
@@ -130,12 +130,12 @@ void add_nsap(AsnOid *netprefix, AsnOid *esi)
   if ((fd = socket(AF_ATMSVC, SOCK_DGRAM, 0)) < 0)
     diag(COMPONENT, DIAG_FATAL, "socket: %s", strerror(errno));
 
-  req.number = atm_itf;
+  req.number = itf;
   req.arg = &addr;
   req.length = sizeof(addr);
 
-  if(ioctl(fd, SIOCSIFATMADDR, &req) < 0)
-    diag(COMPONENT, DIAG_FATAL, "ioctl SIOCSIFATMADDR: %s", strerror(errno));
+  if(ioctl(fd, ATM_ADDADDR, &req) < 0)
+    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_ADDADDR: %s", strerror(errno));
 
   close(fd);
 
@@ -215,17 +215,17 @@ int send_message(int fd, Message *message)
 }
 
 
-int open_ilmi(void)
+int open_ilmi(int itf)
 {
     struct sockaddr_atmpvc addr;
-    int fd,len;
+    int fd;
  
     if((fd = socket(PF_ATMPVC, SOCK_DGRAM, ATM_AAL5)) < 0)
       diag(COMPONENT, DIAG_FATAL, "socket: %s", strerror(errno));
 
     memset(&addr, 0, sizeof(addr));
     addr.sap_family = AF_ATMPVC;
-    addr.sap_addr.itf = ATM_ITF_ANY;
+    addr.sap_addr.itf = itf;
     addr.sap_rxtp.class = ATM_UBR;
     addr.sap_txtp.class = ATM_UBR;
     addr.sap_rxtp.max_sdu = MAX_ILMI_MSG;
@@ -236,10 +236,5 @@ int open_ilmi(void)
     if(bind(fd, (struct sockaddr *) &addr, sizeof(addr)) < 0)
       diag(COMPONENT, DIAG_FATAL, "bind: %s", strerror(errno));
 
-    if(getsockname(fd, (struct sockaddr *) &addr, &len) < 0)
-      diag(COMPONENT, DIAG_FATAL, "getsockname: %s", strerror(errno));
-    atm_itf = addr.sap_addr.itf;
-    diag(COMPONENT, DIAG_INFO, "serving ATM interface %d",atm_itf);
-
     return fd;
 }
