@@ -1,9 +1,10 @@
 /* text2atm.c - Converts textual representation of ATM address to binary
 		encoding */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
+#include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include <limits.h>
@@ -174,10 +175,10 @@ static int try_e164(const char *text,struct sockaddr_atmsvc *addr,int flags)
 	}
 	else if (*text != '.') break;
 	    else {
-		if (!dot) return FATAL; /* two dots in a row */
+		if (!dot) return TRY_OTHER; /* two dots in a row */
 		dot = 0;
 	    }
-    if (!dot) return FATAL;
+    if (!dot) return TRY_OTHER;
     addr->sas_addr.pub[i] = 0;
     *addr->sas_addr.prv = 0;
     result = 0;
@@ -194,6 +195,39 @@ static int try_e164(const char *text,struct sockaddr_atmsvc *addr,int flags)
 }
 
 
+static int search(FILE *file,const char *text,struct sockaddr *addr,int length,
+  int flags)
+{
+    char line[MAX_ATM_NAME_LEN+1];
+    const char *here;
+    int result;
+
+    while (fgets(line,MAX_ATM_NAME_LEN,file)) {
+	if (!strtok(line,"\t\n ")) continue;
+	while ((here = strtok(NULL,"\t\n ")))
+	    if (!strcasecmp(here,text)) {
+		here = strtok(line,"\t\n ");
+		result = text2atm(here,addr,length,flags);
+		if (result >= 0) return result;
+	    }
+    }
+    return TRY_OTHER;
+}
+
+
+static int try_name(const char *text,struct sockaddr *addr,int length,
+  int flags)
+{
+    FILE *file;
+    int result;
+
+    if (!(file = fopen(HOSTS_ATM,"r"))) return TRY_OTHER;
+    result = search(file,text,addr,length,flags);
+    (void) fclose(file);
+    return result;
+}
+
+
 int text2atm(const char *text,struct sockaddr *addr,int length,int flags)
 {
     int result;
@@ -205,13 +239,14 @@ int text2atm(const char *text,struct sockaddr *addr,int length,int flags)
 	result = try_pvc(text,(struct sockaddr_atmpvc *) addr,flags);
 	if (result != TRY_OTHER) return result;
     }
-    if (!(flags & T2A_SVC)) return -1;
-    if (length < sizeof(struct sockaddr_atmsvc)) return -1;
-    result = try_nsap(text,(struct sockaddr_atmsvc *) addr,flags);
-    if (result != TRY_OTHER) return result;
-    result = try_e164(text,(struct sockaddr_atmsvc *) addr,flags);
-    if (result != TRY_OTHER) return result;
+    if ((flags & T2A_SVC) && length >= sizeof(struct sockaddr_atmsvc)) {
+	result = try_nsap(text,(struct sockaddr_atmsvc *) addr,flags);
+	if (result != TRY_OTHER) return result;
+	result = try_e164(text,(struct sockaddr_atmsvc *) addr,flags);
+	if (result != TRY_OTHER) return result;
+    }
     if (!(flags & T2A_NAME)) return -1;
-    /* @@@ name resolution */
+    result = try_name(text,addr,length,flags & ~T2A_NAME);
+    if (result != TRY_OTHER) return result;
     return -1;
 }
