@@ -73,9 +73,6 @@ static int send_setup(SOCKET *sock)
 	q_assign(&dsc,QF_cgpn_type,0);
 	q_write(&dsc,QF_cgpn,(void *) local->sas_addr.prv,ATM_ESA_LEN);
     }
-    if (*sock->remote->sas_addr.prv) /* improve @@@ */
-	q_write(&dsc,QF_cdpn_esa,(void *) sock->remote->sas_addr.prv,
-	  ATM_ESA_LEN);
     if (net) {
 	q_assign(&dsc,QF_vpi,0);
 	q_assign(&dsc,QF_vci,assign_vci); /* @@@ */
@@ -102,6 +99,27 @@ static int send_connect(SOCKET *sock)
 }
 
 
+static int bind_check(SOCKET *sock,struct sockaddr_atmsvc *local)
+{
+    struct sockaddr_atmsvc *use;
+
+    if (sock->local && (*sock->local->sas_addr.prv ||
+      *sock->local->sas_addr.pub)) return 0;
+    if (*local->sas_addr.prv || *local->sas_addr.pub) use = local;
+    else {
+	if (local_addr[0].state != ls_same) {
+	    diag(COMPONENT,DIAG_ERROR,"no local address");
+	    return -EADDRNOTAVAIL;
+	}
+	use = &local_addr[0].addr;
+    }
+    use->sas_addr.blli = NULL; /* paranoia */
+    if (sock->local) *sock->local = *use;
+    else sock->local = sap_copy(use);
+    return 0;
+}
+
+
 static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 {
     SOCKET *next;
@@ -113,7 +131,8 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 	    if (!*msg->svc.sas_addr.pub && !*msg->svc.sas_addr.prv)
 		if (local_addr[0].state != ls_same)
 		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
-		else send_kernel(msg->vcc,0,as_okay,0,NULL,&local_addr[0].addr);
+		else send_kernel(msg->vcc,0,as_okay,0,NULL,&local_addr[0].addr,
+		      NULL);
 	    else {
 		LOCAL_ADDR *walk;
 
@@ -122,19 +141,21 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		      &msg->svc,0,0)) break;
 		if (walk->state == ls_unused)
 		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
-		else send_kernel(msg->vcc,0,as_okay,0,NULL,NULL);
+		else send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,NULL);
 	    }
 	    return;
 	case as_establish: /* NULL or LISTENING */
 	    if (sock && sock->state != ss_listening) break;
 	    if (!sock) {
 		sock = new_sock(msg->vcc);
-		sock->local = *msg->local.sas_addr.prv || *msg->local.sas_addr.
-		  pub ? sap_copy(&msg->local) : NULL;
-		sock->remote = sap_copy(&msg->svc);
-		if (!sock->remote) {
+		error = bind_check(sock,&msg->local);
+		if (!error) {
+		    sock->remote = sap_copy(&msg->svc);
+		    if (!sock->remote) error = -EINVAL;
+		}
+		if (error) {
 		    free_sock(sock);
-		    SEND_ERROR(msg->vcc,-EINVAL);
+		    SEND_ERROR(msg->vcc,error);
 		    return;
 		}
 		sock->pvc.sap_txtp = sock->remote->sas_txtp;
@@ -193,7 +214,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		}
 		sock = new_sock(msg->vcc);
 		sock->local = sap_copy(&msg->svc);
-		send_kernel(sock->id,0,as_okay,0,NULL,NULL);
+		send_kernel(sock->id,0,as_okay,0,NULL,NULL,NULL);
 		sock->state = ss_listening;
 		return;
 	    }
@@ -269,6 +290,7 @@ void itf_load(int itf)
 		case ls_removed:
 		    diag(COMPONENT,DIAG_INFO,"Removed local ATM address %s at "
 		      "itf %d",buf,itf);
+		    walk->state = ls_unused;
 		    /* @@@ delete SVCs using that address ? */
 		    break;
 		default:
