@@ -61,7 +61,7 @@ extern int pretty;
   (int) sizeof(struct atm_blli)+1)
 #define bllis2msg(n) (sizeof(struct atmsvc_msg)+sizeof(struct atm_blli)*((n)-1))
 
-#define SEND_ERROR(vcc,code) send_kernel(vcc,0L,as_close,code,NULL,NULL)
+#define SEND_ERROR(vcc,code) send_kernel(vcc,0L,as_close,code,NULL,NULL,NULL)
 
 
 void from_kernel(struct atmsvc_msg *msg,int size);
@@ -70,7 +70,7 @@ void itf_load(int itf);
 void to_q2931(void *msg,int size);
 void send_kernel(unsigned long vcc,unsigned long listen_vcc,
   enum atmsvc_msg_type type,int reply,struct sockaddr_atmpvc *pvc,
-  struct sockaddr_atmsvc *svc);
+  struct sockaddr_atmsvc *svc,struct sockaddr_atmsvc *local);
 void from_net(void *msg,int size);
 void to_signaling(void *msg,int size);
 void saal_failure(void);
