@@ -105,7 +105,7 @@ static void setup_call(unsigned long call_ref)
 	  pretty) < 0) strcpy(buffer,"<invalid address>");
 	diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
     }
-    send_kernel(0,sock->id,as_indicate,0,NULL,this->remote);
+    send_kernel(0,sock->id,as_indicate,0,&this->pvc,this->remote,NULL);
     for (walk = &sock->listen; *walk; walk = &(*walk)->listen);
     *walk = this;
     diag(COMPONENT,DIAG_DEBUG,"SE vpi.vci=%d.%d",this->pvc.sap_addr.vpi,
@@ -274,7 +274,7 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    send_connect_ack(sock);
 	    /* @@@ fill in sock->remote */
 	    /* @@@ fill in traffic parameters */
-	    send_kernel(sock->id,0,as_okay,0,&sock->pvc,NULL);
+	    send_kernel(sock->id,0,as_okay,0,&sock->pvc,NULL,sock->local);
 	    new_state(sock,ss_connected);
 	    diag(COMPONENT,DIAG_INFO,"Active open succeeded (CR 0x%06X)",
 	      sock->call_ref);
@@ -288,7 +288,7 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    }
 	    if (sock->state != ss_accepting) break;
 	    STOP_TIMER(sock);
-	    send_kernel(sock->id,0,as_okay,0,NULL,NULL);
+	    send_kernel(sock->id,0,as_okay,0,NULL,NULL,sock->local);
 	    new_state(sock,ss_connected);
 	    diag(COMPONENT,DIAG_INFO,"Passive open succeeded (CR 0x%06X)",
 	      sock->call_ref);
