@@ -172,6 +172,8 @@ int sap_compat(const struct sockaddr_atmsvc *old,
     struct sockaddr_atmsvc *res;
     int compat;
 
+    if ((*old->sas_addr.prv || *old->sas_addr.pub) && !atm_equal(old,new,0,0))
+	return 0;
     if (!out) res = NULL;
     else res = alloc(sizeof(struct sockaddr_atmsvc)+sizeof(struct atm_blli));
     compat = do_sap_compat(old,new,res);
@@ -191,7 +193,11 @@ void sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
 {
     struct atm_blli *blli;
 
+    if (*sap->sas_addr.prv) /* improve @@@ */
+	q_write(dsc,QF_cdpn_esa,(void *) sap->sas_addr.prv,ATM_ESA_LEN);
     switch (sap->sas_txtp.class) {
+	case ATM_NONE:
+	    break;
 	case ATM_UBR:
 	    q_assign(dsc,QF_fw_pcr_01,ATM_OC3_PCR);
 	    q_assign(dsc,QF_bw_pcr_01,ATM_OC3_PCR);
@@ -199,10 +205,18 @@ void sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap)
 	    break;
 	case ATM_CBR:
 	    q_assign(dsc,QF_fw_pcr_01,sap->sas_txtp.min_pcr);
+	    break;
+        default:
+	    diag(COMPONENT,DIAG_FATAL,"bad TX class (%d)",sap->sas_txtp.class);
+    }
+    switch (sap->sas_txtp.class == ATM_UBR ? ATM_NONE : sap->sas_rxtp.class) {
+	case ATM_NONE:
+	    break;
+	case ATM_CBR:
 	    q_assign(dsc,QF_bw_pcr_01,sap->sas_rxtp.min_pcr);
 	    break;
         default:
-	    diag(COMPONENT,DIAG_FATAL,"bad class (%d)",sap->sas_txtp.class);
+	    diag(COMPONENT,DIAG_FATAL,"bad RX class (%d)",sap->sas_txtp.class);
     }
     if (sap->sas_txtp.max_sdu)
 	q_assign(dsc,QF_fw_max_sdu,sap->sas_txtp.max_sdu);
@@ -300,6 +314,8 @@ struct sockaddr_atmsvc *sap_decode(Q_DSC *dsc)
     blli = (struct atm_blli *) (sap+1);
     memset(sap,0,sizeof(struct sockaddr_atmsvc));
     sap->sas_family = AF_ATMSVC;
+    if (q_present(dsc,QF_cdpn_esa)) /* @@@ handle E.164 too */
+	(void) q_read(dsc,QF_cdpn_esa,(void *) &sap->sas_addr.prv,ATM_ESA_LEN);
     if (q_present(dsc,QF_aal_type))
 	if (q_fetch(dsc,QF_aal_type) != 5)
 	    diag(COMPONENT,DIAG_ERROR,"AAL type %d requested",
