@@ -1,6 +1,6 @@
 /* sscop.c - SSCOP (Q.2110) protocol */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1996 by Werner Almesberger, EPFL-LRC */
  
 
 #if 1 /* debugging only */
@@ -699,7 +699,13 @@ static void set_data_transfer_timers(SSCOP_DSC *dsc) /* 20.51 */
 
 static void initialize_vr_mr(SSCOP_DSC *dsc)
 {
-    dsc->vr_mr = SSCOP_CF_MR;
+    dsc->vr_mr = MOD24(dsc->vr_h+SSCOP_CF_MR);
+}
+
+
+static void update_vr_mr(SSCOP_DSC *dsc)
+{
+    initialize_vr_mr(dsc);
 }
 
 
@@ -935,6 +941,7 @@ static void data_sd(SSCOP_DSC *dsc,int s,void *msg,int length) /* 20.38-39 */
 	if (dsc->ops->data_ind) dsc->ops->data_ind(dsc->user,msg,length,s);
 	if (s == dsc->vr_h) {
 	    dsc->vr_r = dsc->vr_h = MOD24(s+1);
+	    update_vr_mr(dsc);
 	    return;
 	}
 	while (1) {
@@ -953,12 +960,14 @@ static void data_sd(SSCOP_DSC *dsc,int s,void *msg,int length) /* 20.38-39 */
     if (s == dsc->vr_h) {
 	queue_put(&dsc->rx_buf,buf);
 	INC24(dsc->vr_h);
+	update_vr_mr(dsc);
 	return;
     }
     if (NORM_RX(dsc->vr_h) < NORM_RX(s)) {
 	queue_put(&dsc->rx_buf,buf);
 	send_ustat(dsc,dsc->vr_h,s);
 	dsc->vr_h = MOD24(s+1);
+	update_vr_mr(dsc);
 	return;
     }
     if (queue_lookup(&dsc->rx_buf,s)) queue_put(&dsc->rx_buf,buf);
@@ -985,6 +994,7 @@ static void data_poll(SSCOP_DSC *dsc,int s) /* 20.41-42 */
 	return;
     }
     dsc->vr_h = NORM_RX(dsc->vr_mr) < NORM_RX(s) ? dsc->vr_mr : s;
+    update_vr_mr(dsc);
     /* K */
     curr = 0;
     i = dsc->vr_r;
@@ -993,17 +1003,17 @@ static void data_poll(SSCOP_DSC *dsc,int s) /* 20.41-42 */
 	    INC24(i);
 	    continue;
 	}
-	dsc->list[curr++] = i;
+	dsc->list[curr++] = htonl(i);
 	if (curr >= dsc->cf_max_stat) {
 	    send_pdu(dsc,SSCOP_STAT,dsc->list,curr*4);
 	    curr = 0;
-	    dsc->list[curr++] = i;
+	    dsc->list[curr++] = htonl(i);
 	}
 	do INC24(i);
 	while (i != dsc->vr_h && !queue_lookup(&dsc->rx_buf,i));
-	dsc->list[curr++] = i;
+	dsc->list[curr++] = htonl(i);
     }
-    if (NORM_RX(i) > NORM_RX(dsc->vr_h)) dsc->list[curr++] = i;
+    if (NORM_RX(i) > NORM_RX(dsc->vr_h)) dsc->list[curr++] = htonl(i);
     send_pdu(dsc,SSCOP_STAT,dsc->list,curr*4);
 }
 
