@@ -6,10 +6,12 @@
 #include <sys/socket.h>
 #include <linux/atm.h>
 
+#include "atm.h"
+
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"usage: %s [ <nsap_addr> ]\n",name);
+    fprintf(stderr,"usage: %s [ <atm_addr> ]\n",name);
     exit(1);
 }
 
@@ -18,36 +20,36 @@ int main(int argc,char **argv)
 {
     struct sockaddr_atmsvc addr;
     struct atm_blli blli;
+    char buffer[MAX_ATM_ADDR_LEN+1];
     char *here;
-    int s,n,i,passive;
+    int s,passive,len;
 
-    if ((s = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0) {
+    if ((s = socket(PF_ATMSVC,SOCK_DGRAM,ATM_AAL5)) < 0) {
 	perror("socket");
 	return 1;
     }
     memset(&addr,0,sizeof(addr));
     memset(&blli,0,sizeof(blli));
-    if (!(passive = argc == 1)) {
-	if (argc != 2) usage(argv[0]);
-	if (strlen(argv[1]) != 40) {
-	    fprintf(stderr,"NSAP address must contain exactly 40 hex digits\n");
-	    return 1;
-	}
-	for (i = 0; i < 20; i++) {
-	    int tmp;
-
-	    sscanf(argv[1]+2*i,"%2x",&tmp);
-	    addr.sas_addr.prv[i] = tmp;
-	}
-    }
+    addr.sas_family = AF_ATMSVC;
     addr.sas_family = AF_ATMSVC;
     addr.sas_addr.blli = &blli;
+    addr.sas_txtp.class = addr.sas_rxtp.class = ATM_UBR;
     blli.l2_proto = 1;
     blli.next = NULL;
     if (bind(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	perror("bind");
 	return 1;
     }
+    if (!(passive = argc == 1)) {
+	if (argc != 2) usage(argv[0]);
+	if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),T2A_NAME)
+	  < 0) {
+	    fprintf(stderr,"%s: invalid ATM address\n",argv[1]);
+	    return 1;
+	}
+    }
+    addr.sas_addr.blli = &blli;
+    addr.sas_txtp.class = addr.sas_rxtp.class = ATM_UBR;
     here = strrchr(argv[0],'/');
     if (!passive) {
 	if (connect(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
@@ -60,11 +62,28 @@ int main(int argc,char **argv)
 	    perror("listen");
 	    return 1;
 	}
-	if ((n = accept(s,NULL,0)) < 0) {
+	if ((s = accept(s,NULL,0)) < 0) { /* loses listening socket dsc */
 	    perror("accept");
 	    return 1;
 	}
     }
+    len = sizeof(addr);
+    if (getsockname(s,(struct sockaddr *) &addr,&len) < 0)
+	perror("getsockname");
+    else {
+	if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,
+	  A2T_NAME | A2T_PRETTY) < 0) strcpy(buffer,"<invalid>");
+	printf("Local: %s\n",buffer);
+    }
+    fflush(stdout);
+    len = sizeof(addr);
+    if (getpeername(s,(struct sockaddr *) &addr,&len) < 0)
+	perror("getpeername");
+    else {
+	if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,
+	  A2T_NAME | A2T_PRETTY) < 0) strcpy(buffer,"<invalid>");
+	printf("Remote: %s\n",buffer);
+    }
     printf("Press [Enter] to continue ");
     fflush(stdout);
     (void) getchar();
